@@ -274,7 +274,7 @@ class HangmanSolver(HTMLRequestHandler):
         await self.render("pages/hangman_solver.html", **asdict(hangman))
 
 
-class HangmanSolverAPI(HangmanSolver, APIRequestHandler):
+class HangmanSolverAPI(APIRequestHandler, HangmanSolver):
     """Request handler for the hangman solver JSON API."""
 
     RATELIMIT_GET_LIMIT = 10
