@@ -88,9 +88,18 @@ class LOLWUT(HTMLRequestHandler):
 class LOLWUTAPI(APIRequestHandler):
     """The request handler for the LOLWUT API."""
 
+    POSSIBLE_CONTENT_TYPES: tuple[str, ...] = (
+        "text/plain",
+        *APIRequestHandler.POSSIBLE_CONTENT_TYPES,
+    )
+
     async def get(self, args: None | str = None, *, head: bool = False) -> None:
         """Handle GET requests to the LOLWUT API."""
         if not EVENT_REDIS.is_set():
             raise HTTPError(503)
-        self.set_header("Content-Type", "text/plain; charset=utf-8")
-        await self.finish(await generate_art(self.redis, args, head))
+        art = await generate_art(self.redis, args, head)
+        if self.content_type == "text/plain":
+            self.set_header("Content-Type", "text/plain; charset=UTF-8")
+            return await self.finish(art)
+
+        await self.finish({"LOLWUT": art})
