@@ -21,20 +21,21 @@ import os
 import random
 import sys
 import time
-from collections.abc import Awaitable, Callable, Iterable
+from collections.abc import Callable, Iterable
 from dataclasses import dataclass
 from datetime import date, datetime
-from typing import Any, Literal
+from multiprocessing import Value
+from typing import Any, Literal, cast
 
 import elasticapm  # type: ignore
 import orjson as json
-import tornado.web
-from redis.asyncio import Redis  # type: ignore
+from redis.asyncio import Redis
 from tornado.httpclient import AsyncHTTPClient
-from tornado.web import HTTPError
+from tornado.web import Application, HTTPError
+from UltraDict import UltraDict  # type: ignore
 
 from .. import DIR as ROOT_DIR
-from .. import ORJSON_OPTIONS
+from .. import EVENT_REDIS, ORJSON_OPTIONS
 from ..utils.request_handler import HTMLRequestHandler
 from ..utils.utils import emojify
 
@@ -42,14 +43,16 @@ DIR = os.path.dirname(__file__)
 
 logger = logging.getLogger(__name__)
 
-API_URL: str = "https://zitate.prapsschnalinen.de/api"
+API_URL = "https://zitate.prapsschnalinen.de/api"
 
-QUOTES_CACHE: dict[int, Quote] = {}
-AUTHORS_CACHE: dict[int, Author] = {}
-WRONG_QUOTES_CACHE: dict[tuple[int, int], WrongQuote] = {}
+# fmt: off
+QUOTES_CACHE: dict[int, Quote] = UltraDict(buffer_size=16384)
+AUTHORS_CACHE: dict[int, Author] = UltraDict(buffer_size=16384)
+WRONG_QUOTES_CACHE: dict[tuple[int, int], WrongQuote] = UltraDict(buffer_size=16384)
+# fmt: on
 
-MAX_QUOTES_ID: list[int] = [0]
-MAX_AUTHORS_ID: list[int] = [0]
+MAX_QUOTES_ID = Value("Q", 0)
+MAX_AUTHORS_ID = Value("Q", 0)
 
 
 @dataclass
@@ -78,6 +81,7 @@ class Author(QuotesObjBase):
 
     def update_name(self, name: str) -> None:
         """Update author data with another author."""
+        name = fix_author_name(name)
         if self.name != name:
             # name changed -> info should change too
             self.info = None
@@ -122,7 +126,7 @@ class Quote(QuotesObjBase):
         self, quote: str, author_id: int, author_name: str
     ) -> None:
         """Update quote data with new data."""
-        self.quote = quote
+        self.quote = fix_quote_str(quote)
         if self.author.id == author_id:
             self.author.update_name(author_name)
             return
@@ -178,18 +182,18 @@ class WrongQuote(QuotesObjBase):
 
     async def fetch_new_data(self) -> WrongQuote:
         """Fetch new data from the API."""
-        return parse_wrong_quote(
-            (
-                await make_api_request(
-                    "wrongquotes",
-                    f"quote={self.quote.id}"
-                    f"&author={self.author.id}"
-                    "&simulate=true",
-                )
-            )[0]
-            if self.id == -1
-            else await make_api_request(f"wrongquotes/{self.id}")
-        )
+        if self.id == -1:
+            api_data = await make_api_request(
+                "wrongquotes",
+                f"quote={self.quote.id}&author={self.author.id}&simulate=true",
+            )
+            if api_data:
+                api_data = api_data[0]
+        else:
+            api_data = await make_api_request(f"wrongquotes/{self.id}")
+        if not api_data:
+            return self
+        return parse_wrong_quote(api_data)
 
     async def vote(
         self, vote: Literal[-1, 1], fast: bool = False
@@ -235,6 +239,7 @@ class WrongQuote(QuotesObjBase):
 
 def get_wrong_quotes(
     filter_fun: None | Callable[[WrongQuote], bool] = None,
+    *,
     sort: bool = False,  # sorted by rating
     filter_real_quotes: bool = True,
     shuffle: bool = False,
@@ -322,9 +327,17 @@ async def make_api_request(
     return json.loads(response.body)
 
 
+def fix_author_name(name: str) -> str:
+    """Fix common mistakes in authors."""
+    if len(name) > 2 and name.startswith("(") and name.endswith(")"):
+        # remove () from author name, that shouldn't be there
+        name = name[1:-1]
+    return name.strip()
+
+
 def get_author_updated_with(author_id: int, author_name: str) -> Author:
     """Get the author with the given id and the name."""
-    author_name = author_name.strip()
+    author_name = fix_author_name(author_name)
     if not author_name:
         author_name = "None"
     author = AUTHORS_CACHE.setdefault(
@@ -332,7 +345,7 @@ def get_author_updated_with(author_id: int, author_name: str) -> Author:
         Author(author_id, author_name),
     )
     author.update_name(author_name)
-    MAX_AUTHORS_ID[0] = max(MAX_AUTHORS_ID[0], author_id)
+    MAX_AUTHORS_ID.value = max(MAX_AUTHORS_ID.value, author_id)
     return author
 
 
@@ -345,11 +358,10 @@ def fix_quote_str(quote_str: str) -> str:
     """Fix common mistakes in quotes."""
     if (
         len(quote_str) > 2
-        and (quote_str.startswith('"') or quote_str.startswith("„"))
-        and (quote_str.endswith('"') or quote_str.endswith("“"))
+        and quote_str.startswith(('"', "„", "“"))
+        and quote_str.endswith(('"', "“", "”"))
     ):
-        # remove double quotes from quote,
-        # that are stupid and shouldn't be there
+        # remove quotation marks from quote, that shouldn't be there
         quote_str = quote_str[1:-1]
 
     return quote_str.strip()
@@ -370,13 +382,14 @@ def parse_quote(json_data: dict[str, Any]) -> Quote:
         ),
     )
 
-    MAX_QUOTES_ID[0] = max(MAX_QUOTES_ID[0], quote.id)
+    MAX_QUOTES_ID.value = max(MAX_QUOTES_ID.value, quote.id)
 
     quote.update_quote(
         quote_str,
         author.id,
         author.name,
     )
+
     return quote
 
 
@@ -418,112 +431,133 @@ def parse_list_of_quote_data(
     return tuple(parse_fun(json_data) for json_data in _json_list)
 
 
-async def update_cache_periodically(
-    app: tornado.web.Application,
-    setup_redis_awaitable: None | Awaitable[Any] = None,
-) -> None:
+async def update_cache_periodically(app: Application) -> None:  # noqa: C901
     """Start updating the cache every hour."""
-    if setup_redis_awaitable:
-        await setup_redis_awaitable
-    redis: None | Redis = app.settings.get("REDIS")
+    # pylint: disable=too-complex, too-many-branches
+    try:
+        await asyncio.wait_for(EVENT_REDIS.wait(), 5)
+    except asyncio.TimeoutError:
+        pass
+    redis: Redis = cast(Redis, app.settings.get("REDIS"))  # type: ignore[type-arg]
     prefix: str = app.settings.get("REDIS_PREFIX", "")
-    if redis:
+    apm: None | elasticapm.Client
+    if EVENT_REDIS.is_set():  # pylint: disable=too-many-nested-blocks
         parse_list_of_quote_data(
-            await redis.get(f"{prefix}:cached-quote-data:wrongquotes"),
+            await redis.get(f"{prefix}:cached-quote-data:wrongquotes"),  # type: ignore[arg-type]  # noqa: B950  # pylint: disable=line-too-long, useless-suppression
             parse_wrong_quote,
         )
         parse_list_of_quote_data(
-            await redis.get(f"{prefix}:cached-quote-data:quotes"),
+            await redis.get(f"{prefix}:cached-quote-data:quotes"),  # type: ignore[arg-type]  # noqa: B950  # pylint: disable=line-too-long, useless-suppression
             parse_quote,
         )
         parse_list_of_quote_data(
-            await redis.get(f"{prefix}:cached-quote-data:authors"),
+            await redis.get(f"{prefix}:cached-quote-data:authors"),  # type: ignore[arg-type]  # noqa: B950  # pylint: disable=line-too-long, useless-suppression
             parse_author,
         )
-    if redis and QUOTES_CACHE and AUTHORS_CACHE and WRONG_QUOTES_CACHE:
-        last_update = await redis.get(f"{prefix}:cached-quote-data:last-update")
-        if last_update:
-            last_update_int = int(last_update)
-            since_last_update = int(time.time()) - last_update_int
-            if 0 <= since_last_update < 60 * 60:
-                # wait until the last update is at least one hour old
-                update_cache_in = 60 * 60 - since_last_update
-                if not sys.flags.dev_mode and update_cache_in > 60:
-                    # if in production mode update wrong quotes just to be sure
-                    await update_cache(
-                        app, update_quotes=False, update_authors=False
+        if QUOTES_CACHE and AUTHORS_CACHE and WRONG_QUOTES_CACHE:
+            last_update = await redis.get(
+                f"{prefix}:cached-quote-data:last-update"
+            )
+            if last_update:
+                last_update_int = int(last_update)
+                since_last_update = int(time.time()) - last_update_int
+                if 0 <= since_last_update < 60 * 60:
+                    # wait until the last update is at least one hour old
+                    update_cache_in = 60 * 60 - since_last_update
+                    if not sys.flags.dev_mode and update_cache_in > 60:
+                        # if in production mode update wrong quotes just to be sure
+                        try:
+                            await update_cache(
+                                app, update_quotes=False, update_authors=False
+                            )
+                        except Exception as exc:  # pylint: disable=broad-except
+                            logger.exception(exc)
+                            logger.error("Updating quotes cache failed.")
+                            apm = app.settings.get("ELASTIC_APM_CLIENT")
+                            if apm:
+                                apm.capture_exception()
+                        else:
+                            logger.info("Updated quotes cache successfully.")
+                    logger.info(
+                        "Next update of quotes cache in %d seconds",
+                        update_cache_in,
                     )
-                logger.info(
-                    "Next update of quotes cache in %d seconds",
-                    update_cache_in,
-                )
-                await asyncio.sleep(update_cache_in)
+                    await asyncio.sleep(update_cache_in)
 
+    failed = 0
     # pylint: disable=while-used
     while True:  # update the cache every hour
-        await update_cache(app)
-        await asyncio.sleep(60 * 60)
+        try:
+            await update_cache(app)
+        except Exception as exc:  # pylint: disable=broad-except
+            logger.exception(exc)
+            logger.error("Updating quotes cache failed.")
+            apm = app.settings.get("ELASTIC_APM_CLIENT")
+            if apm:
+                apm.capture_exception()
+            failed += 1
+            await asyncio.sleep(pow(min(failed * 2, 60), 2))  # 4,16,...,60*60
+        else:
+            logger.info("Updated quotes cache successfully.")
+            failed = 0
+            await asyncio.sleep(60 * 60)
 
 
-async def update_cache(  # noqa: C901  # pylint: disable=too-complex
-    app: tornado.web.Application,
+async def update_cache(
+    app: Application,
     update_wrong_quotes: bool = True,
     update_quotes: bool = True,
     update_authors: bool = True,
 ) -> None:
     """Fill the cache with all data from the API."""
     logger.info("Updating quotes cache...")
-    redis: None | Redis = app.settings.get("REDIS")
+    redis: Redis = cast(Redis, app.settings.get("REDIS"))  # type: ignore[type-arg]
     prefix: str = app.settings.get("REDIS_PREFIX", "")
-    try:  # pylint: disable=too-many-try-statements
+    redis_available = EVENT_REDIS.is_set()
 
-        if update_wrong_quotes:
-            parse_list_of_quote_data(
-                wq_data := await make_api_request("wrongquotes"),
-                parse_wrong_quote,
-            )
-            if wq_data and redis:
-                await redis.set(
-                    f"{prefix}:cached-quote-data:wrongquotes",
-                    json.dumps(wq_data, option=ORJSON_OPTIONS),
-                )
-
-        if update_quotes:
-            parse_list_of_quote_data(
-                quotes_data := await make_api_request("quotes"),
-                parse_quote,
-            )
-            if quotes_data and redis:
-                await redis.set(
-                    f"{prefix}:cached-quote-data:quotes",
-                    json.dumps(quotes_data, option=ORJSON_OPTIONS),
-                )
-
-        if update_authors:
-            parse_list_of_quote_data(
-                authors_data := await make_api_request("authors"),
-                parse_author,
-            )
-            if authors_data and redis:
-                await redis.set(
-                    f"{prefix}:cached-quote-data:authors",
-                    json.dumps(authors_data, option=ORJSON_OPTIONS),
-                )
-
-        if redis and update_wrong_quotes and update_quotes and update_authors:
+    if update_wrong_quotes:
+        parse_list_of_quote_data(
+            wq_data := await make_api_request("wrongquotes"),
+            parse_wrong_quote,
+        )
+        if wq_data and redis_available:
             await redis.set(
-                f"{prefix}:cached-quote-data:last-update",
-                int(time.time()),
+                f"{prefix}:cached-quote-data:wrongquotes",
+                json.dumps(wq_data, option=ORJSON_OPTIONS),
             )
 
-    except Exception as exc:  # pylint: disable=broad-except
-        logger.exception(exc)
-        logger.error("Updating quotes cache failed.")
-        apm: None | elasticapm.Client = app.settings.get("ELASTIC_APM_CLIENT")
-        if apm:
-            apm.capture_exception()
-    else:
-        logger.info("Updated quotes cache successfully.")
+    if update_quotes:
+        parse_list_of_quote_data(
+            quotes_data := await make_api_request("quotes"),
+            parse_quote,
+        )
+        if quotes_data and redis_available:
+            await redis.set(
+                f"{prefix}:cached-quote-data:quotes",
+                json.dumps(quotes_data, option=ORJSON_OPTIONS),
+            )
+
+    if update_authors:
+        parse_list_of_quote_data(
+            authors_data := await make_api_request("authors"),
+            parse_author,
+        )
+        if authors_data and redis_available:
+            await redis.set(
+                f"{prefix}:cached-quote-data:authors",
+                json.dumps(authors_data, option=ORJSON_OPTIONS),
+            )
+
+    if (
+        redis_available
+        and update_wrong_quotes
+        and update_quotes
+        and update_authors
+    ):
+        await redis.set(
+            f"{prefix}:cached-quote-data:last-update",
+            int(time.time()),
+        )
 
 
 async def get_author_by_id(author_id: int) -> Author:
@@ -531,7 +565,6 @@ async def get_author_by_id(author_id: int) -> Author:
     author = AUTHORS_CACHE.get(author_id, None)
     if author is not None:
         return author
-
     return parse_author(await make_api_request(f"authors/{author_id}"))
 
 
@@ -548,7 +581,6 @@ async def get_wrong_quote(
 ) -> WrongQuote:
     """Get a wrong quote with a quote id and an author id."""
     wrong_quote = WRONG_QUOTES_CACHE.get((quote_id, author_id), None)
-
     if wrong_quote:
         if use_cache:
             return wrong_quote
@@ -582,12 +614,12 @@ async def get_rating_by_id(quote_id: int, author_id: int) -> int:
 
 def get_random_quote_id() -> int:
     """Get random quote id."""
-    return random.randint(1, MAX_QUOTES_ID[0])
+    return random.randint(1, MAX_QUOTES_ID.value)
 
 
 def get_random_author_id() -> int:
     """Get random author id."""
-    return random.randint(1, MAX_AUTHORS_ID[0])
+    return random.randint(1, MAX_AUTHORS_ID.value)
 
 
 def get_random_id() -> tuple[int, int]:
