@@ -205,11 +205,12 @@ class QuoteOfTheDayBaseHandler(QuoteReadyCheckHandler):
 class QuoteOfTheDayRss(QuoteOfTheDayBaseHandler):
     """The request handler for the quote of the day RSS feed."""
 
+    POSSIBLE_CONTENT_TYPES = ("application/rss+xml", "application/xml")
     IS_NOT_HTML = True
 
     async def get(self, *, head: bool = False) -> None:
         """Handle GET requests."""
-        self.set_header("Content-Type", "application/rss+xml; charset=UTF-8")
+        self.set_header("Content-Type", f"{self.content_type}; charset=UTF-8")
         if head:
             return
         today = datetime.now(tz=timezone.utc).date()
@@ -226,7 +227,7 @@ class QuoteOfTheDayRss(QuoteOfTheDayBaseHandler):
         )
 
 
-class QuoteOfTheDayAPI(QuoteOfTheDayBaseHandler, APIRequestHandler):
+class QuoteOfTheDayAPI(APIRequestHandler, QuoteOfTheDayBaseHandler):
     """Handler for the JSON API that returns the quote of the day."""
 
     async def get(
@@ -280,7 +281,7 @@ class QuoteOfTheDayRedirect(QuoteOfTheDayBaseHandler):
 
         self.redirect(
             self.fix_url(
-                wrong_quote_data.get_quote_url(), as_json=self.get_as_json()
+                wrong_quote_data.get_quote_url(),
             ),
             False,
         )
