@@ -25,7 +25,7 @@ import random
 import re
 from asyncio import Task
 from collections.abc import Coroutine, Generator
-from typing import Any, Literal, cast
+from typing import Any, Literal
 
 from tornado.web import HTTPError, RedirectHandler
 
@@ -76,13 +76,13 @@ def get_module_info() -> ModuleInfo:
                 RedirectHandler,
                 {"url": "/zitate/{0}-{1}"},
             ),
-            (  # redirect to the new URL (changed because of robots.txt)
-                r"/zitate/([0-9]{1,10})-([0-9]{1,10})/image.([a-zA-Z]{3,4})",
+            (  # redirect to the new URL
+                r"/zitate/([0-9]{1,10})-([0-9]{1,10})/image\.([a-zA-Z]{3,4})",
                 RedirectHandler,
                 {"url": "/zitate/{0}-{1}.{2}"},
             ),
             (
-                r"/zitate/([0-9]{1,10})-([0-9]{1,10}).([a-zA-Z]{3,4})",
+                r"/zitate/([0-9]{1,10})-([0-9]{1,10})\.([a-zA-Z]{3,4})",
                 QuoteAsImage,
             ),
             (  # redirect to the new URL (changed because of robots.txt)
@@ -252,8 +252,8 @@ class QuoteMainPage(QuoteBaseHandler, QuoteOfTheDayBaseHandler):
         """Render the main quote page, with a few links."""
         if head:
             return
-        quote = cast(str, self.get_argument("quote", ""))
-        author = cast(str, self.get_argument("author", ""))
+        quote = self.get_argument("quote", "")
+        author = self.get_argument("author", "")
         if (quote or author) and re.fullmatch(r"^[0-9]+$", quote + author):
             self.redirect(self.fix_url(f"/zitate/{quote}-{author}"))
             return
@@ -289,7 +289,7 @@ class QuoteMainPage(QuoteBaseHandler, QuoteOfTheDayBaseHandler):
         return self.fix_url(f"/zitate/{quote_id}-{author_id}", r=rating_param)
 
 
-class QuoteRedirectAPI(QuoteBaseHandler, APIRequestHandler):
+class QuoteRedirectAPI(APIRequestHandler, QuoteBaseHandler):
     """Redirect to the api for a random quote."""
 
     async def get(  # pylint: disable=unused-argument
@@ -300,7 +300,6 @@ class QuoteRedirectAPI(QuoteBaseHandler, APIRequestHandler):
         return self.redirect(
             self.fix_url(
                 f"/api/zitate/{quote_id}-{author_id}{suffix}",
-                as_json=self.get_as_json(),
             )
         )
 
@@ -328,7 +327,6 @@ class QuoteById(QuoteBaseHandler):
             return self.redirect(
                 self.fix_url(
                     f"/zitate/{wqs[0].quote.id}-{wqs[0].author.id}",
-                    as_json=self.get_as_json(),
                 )
             )
         if head:
@@ -475,7 +473,7 @@ class QuoteById(QuoteBaseHandler):
         return None
 
 
-class QuoteAPIHandler(QuoteById, APIRequestHandler):
+class QuoteAPIHandler(APIRequestHandler, QuoteById):
     """API request handler for the quotes page."""
 
     RATELIMIT_GET_PERIOD = 10
