@@ -28,7 +28,8 @@ import time
 import traceback
 import uuid
 from asyncio import Future
-from collections.abc import Awaitable, Coroutine
+from base64 import b64decode
+from collections.abc import Awaitable, Callable, Coroutine
 from datetime import date, datetime, timedelta, timezone, tzinfo
 from http.client import responses
 from typing import Any, cast
@@ -36,7 +37,10 @@ from urllib.parse import SplitResult, quote, unquote, urlsplit, urlunsplit
 from zoneinfo import ZoneInfo
 
 import elasticapm  # type: ignore
+import html2text
 import orjson as json
+import yaml
+from accept_types import get_best_match  # type: ignore
 from ansi2html import Ansi2HTMLConverter  # type: ignore
 from blake3 import blake3  # type: ignore
 from bs4 import BeautifulSoup
@@ -46,10 +50,9 @@ from elasticsearch.exceptions import ElasticsearchException
 from Levenshtein import distance  # type: ignore
 from redis.asyncio import Redis
 from sympy.ntheory import isprime
-from tornado import web
 from tornado.web import HTTPError, MissingArgumentError, RequestHandler
 
-from .. import EVENT_ELASTICSEARCH, EVENT_REDIS, REPO_URL
+from .. import EVENT_ELASTICSEARCH, EVENT_REDIS, ORJSON_OPTIONS, REPO_URL
 from .static_file_handling import fix_static_url
 from .utils import (
     THEMES,
@@ -87,6 +90,7 @@ class BaseRequestHandler(RequestHandler):
     """The base Tornado request handler used by every page and API."""
 
     REQUIRED_PERMISSION: Permissions = Permissions(0)
+    POSSIBLE_CONTENT_TYPES: tuple[str, ...] = ()
     # the following should be False on security relevant endpoints
     SUPPORTS_COOKIE_AUTHORIZATION: bool = True
     ALLOWED_METHODS: tuple[str, ...] = ("GET",)
@@ -98,9 +102,12 @@ class BaseRequestHandler(RequestHandler):
 
     module_info: ModuleInfo
     # info about page, can be overridden in module_info
-    title = "Das Asoziale Netzwerk"
-    short_title = "Asoziales Netzwerk"
-    description = "Die tolle Webseite des Asozialen Netzwerkes"
+    title: str = "Das Asoziale Netzwerk"
+    short_title: str = "Asoziales Netzwerk"
+    description: str = "Die tolle Webseite des Asozialen Netzwerkes"
+    content_type: str = ""
+
+    _active_origin_trials: set[str]
 
     def initialize(
         self,
@@ -157,7 +164,8 @@ class BaseRequestHandler(RequestHandler):
 
     def set_default_headers(self) -> None:
         """Set default headers."""
-        # see: dev.mozilla.org/docs/Web/HTTP/Headers
+        self._active_origin_trials = set()
+        # dev.mozilla.org/docs/Web/HTTP/Headers
         self.set_header("Access-Control-Max-Age", "7200")
         self.set_header("Access-Control-Allow-Origin", "*")
         self.set_header("Access-Control-Allow-Headers", "*")
@@ -165,10 +173,14 @@ class BaseRequestHandler(RequestHandler):
             "Access-Control-Allow-Methods",
             ", ".join(self.get_allowed_methods()),
         )
-        # Opt out of all FLoC cohort calculation.
+        # opt out of all FLoC cohort calculation
         self.set_header("Permissions-Policy", "interest-cohort=()")
         # don't send the Referer header for cross-origin requests
         self.set_header("Referrer-Policy", "same-origin")
+        # dev.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/SharedArrayBuffer
+        self.set_header("Cross-Origin-Opener-Policy", "same-origin")
+        if self.request.path != "/kaenguru-comics":  # TODO: make this better
+            self.set_header("Cross-Origin-Embedder-Policy", "require-corp")
         if self.settings.get("HSTS"):
             # dev.mozilla.org/docs/Web/HTTP/Headers/Strict-Transport-Security
             self.set_header("Strict-Transport-Security", "max-age=63072000")
@@ -190,6 +202,36 @@ class BaseRequestHandler(RequestHandler):
                         f"X-Permission-{permission.name}",
                         bool_to_str(self.is_authorized(permission)),
                     )
+        self.origin_trial(
+            "AjM7i7vhQFI2RUcab3ZCsJ9RESLDD9asdj0MxpwxHXXtETlsm8dEn+HSd646oPr1dKjn+EcNEj8uV3qFGJzObgsAAAB3eyJvcmlnaW4iOiJodHRwczovL2Fzb3ppYWwub3JnOjQ0MyIsImZlYXR1cmUiOiJTZW5kRnVsbFVzZXJBZ2VudEFmdGVyUmVkdWN0aW9uIiwiZXhwaXJ5IjoxNjg0ODg2Mzk5LCJpc1N1YmRvbWFpbiI6dHJ1ZX0="  # noqa: B950  # pylint: disable=line-too-long, useless-suppression
+        )
+
+    def origin_trial(self, token: str | bytes) -> bool:
+        """Enable an experimental feature."""
+        # pylint: disable=protected-access
+        if token in self._active_origin_trials:
+            return True
+        url = urlsplit(self.request.full_url())
+        payload = json.loads(b64decode(token)[69:])
+        origin = urlsplit(payload["origin"])
+        if url.port is None and url.scheme in {"http", "https"}:
+            url = url._replace(
+                netloc=f"{url.hostname}:{443 if url.scheme == 'https' else 80}"
+            )
+        if self.request._start_time > payload["expiry"]:
+            return False
+        if url.scheme != origin.scheme:
+            return False
+        if url.netloc != origin.netloc and not (
+            payload.get("isSubdomain")
+            and url.netloc.endswith(f".{origin.netloc}")
+        ):
+            return False
+        self.add_header("Origin-Trial", token)
+        self._active_origin_trials.add(
+            token if isinstance(token, str) else token.decode("ascii")
+        )
+        return True
 
     def set_cookie(  # pylint: disable=too-many-arguments
         self,
@@ -216,6 +258,56 @@ class BaseRequestHandler(RequestHandler):
             **kwargs,
         )
 
+    @property
+    def dump(self) -> Callable[..., str | bytes | dict[Any, Any]]:
+        """Get the function used to dump the output dict."""
+        if self.content_type == "application/json":
+            self.set_header("Content-Type", "application/json; charset=UTF-8")
+            return lambda spam: json.dumps(spam, option=ORJSON_OPTIONS)
+
+        if self.content_type == "application/yaml":
+            self.set_header("Content-Type", "application/yaml; charset=UTF-8")
+            return yaml.dump
+
+        return lambda spam: spam  # type: ignore[no-any-return]
+
+    def finish(
+        self, chunk: None | str | bytes | dict[Any, Any] = None
+    ) -> Future[None]:
+        """Finish the request."""
+        if self._finished:
+            return super().finish(chunk)
+
+        if isinstance(chunk, dict):
+            chunk = self.dump(chunk)
+
+        if chunk is not None:
+            self.write(chunk)
+
+        if (
+            "Content-Type" in self._headers
+            and self._headers["Content-Type"].lower().endswith("charset=utf-8")
+            and self._write_buffer
+            and not self._write_buffer[-1].endswith(b"\n")
+        ):
+            self.write(b"\n")
+
+        return super().finish()
+
+    def handle_accept_header(self) -> None:
+        """Handle the Accept header and set `self.content_type`."""
+        if not self.POSSIBLE_CONTENT_TYPES:
+            return
+        content_type = get_best_match(
+            self.request.headers.get("Accept") or "*/*",
+            self.POSSIBLE_CONTENT_TYPES,
+        )
+        if content_type is None:
+            if self.POSSIBLE_CONTENT_TYPES:
+                self.content_type = self.POSSIBLE_CONTENT_TYPES[0]
+            raise HTTPError(406)
+        self.content_type = content_type
+
     async def prepare(  # pylint: disable=invalid-overridden-method
         self,
     ) -> None:
@@ -223,6 +315,8 @@ class BaseRequestHandler(RequestHandler):
         # pylint: disable=attribute-defined-outside-init
         self.now = await self.get_time()
 
+        self.handle_accept_header()
+
         if self.request.method == "GET":
 
             if self.redirect_to_canonical_domain():
@@ -241,7 +335,7 @@ class BaseRequestHandler(RequestHandler):
                     self.request.path,
                     anonymize_ip(str(self.request.remote_ip)),
                 )
-                raise HTTPError(401)
+                raise HTTPError(403)
 
             if not await self.ratelimit(True):
                 await self.ratelimit()
@@ -376,13 +470,20 @@ class BaseRequestHandler(RequestHandler):
             raise HTTPError(405)
         if not self.supports_head():
             raise HTTPError(501)
+
+        if self.content_type == "text/html":
+            self.set_header("Content-Type", "text/html; charset=UTF-8")
+        elif self.content_type:
+            self.set_header("Content-Type", self.content_type)
+            self.dump  # pylint: disable=pointless-statement
+
         kwargs["head"] = True
         return self.get(*args, **kwargs)
 
     # pylint: disable=too-many-return-statements
     def get_error_page_description(self, status_code: int) -> str:
         """Get the description for the error page."""
-        # see: https://developer.mozilla.org/docs/Web/HTTP/Status
+        # https://developer.mozilla.org/docs/Web/HTTP/Status
         if 100 <= status_code <= 199:
             return "Hier gibt es eine total wichtige Information."
         if 200 <= status_code <= 299:
@@ -472,10 +573,6 @@ class BaseRequestHandler(RequestHandler):
         if url.netloc and url.netloc.lower() != self.request.host.lower():
             url = urlsplit(f"/redirect?to={quote(url.geturl())}")
         path = url.path if new_path is None else new_path  # the path of the url
-        # don't add as_json=nope to url if as_json is False
-        # pylint: disable=compare-to-zero  # if None it shouldn't be deleted
-        if "as_json" in query_args and query_args["as_json"] is False:
-            del query_args["as_json"]
         if path.startswith(("/static/", "/soundboard/files/")):
             query_args.update(no_3rd_party=None, theme=None, dynload=None)
         else:
@@ -502,10 +599,6 @@ class BaseRequestHandler(RequestHandler):
             **query_args,
         )
 
-    def get_as_json(self) -> bool:
-        """Get the value of the as_json query parameter."""
-        return self.get_bool_argument("as_json", default=False)
-
     def get_no_3rd_party_default(self) -> bool:
         """Get the default value for the no_3rd_party param."""
         return self.request.host_name.endswith((".onion", ".i2p"))
@@ -580,49 +673,20 @@ class BaseRequestHandler(RequestHandler):
         except ValueError as err:
             raise HTTPError(400, f"{value} is not a Boolean.") from err
 
-    def get_argument(  # type: ignore[override]
-        self,
-        name: str,
-        default: (
-            None | str | web._ArgDefaultMarker
-        ) = web._ARG_DEFAULT,  # pylint: disable=protected-access
-        strip: bool = True,
-    ) -> None | str:
-        """Get an argument based on body or query."""
-        arg = super().get_argument(name, default=None, strip=strip)
-        if arg is not None:
-            return arg
-
-        try:
-            body = json.loads(self.request.body)
-        except Exception:  # pylint: disable=broad-except
-            pass
-        else:
-            if isinstance(body, dict) and name in body:
-                val = str(body[name])
-                return val.strip() if strip else val
-
-        # pylint: disable=protected-access
-        if isinstance(default, web._ArgDefaultMarker):
-            raise web.MissingArgumentError(name)
-
-        return default
-
     def is_authorized(self, permission: Permissions) -> bool:
         """Check whether the request is authorized."""
+        found_keys: list[None | str] = []
         if permission == Permissions(0):
             return True
         api_secrets = self.settings.get("TRUSTED_API_SECRETS", {})
+        found_keys.extend(self.request.headers.get_list("Authorization"))
+        found_keys.extend(self.get_arguments("key"))
+        if self.SUPPORTS_COOKIE_AUTHORIZATION:
+            found_keys.append(self.get_cookie("key", default=None))
         return any(
             permission in api_secrets[key]
-            for key in (
-                self.request.headers.get("Authorization"),
-                self.get_argument("key", default=None),
-                self.get_cookie("key", default=None)
-                if self.SUPPORTS_COOKIE_AUTHORIZATION
-                else None,
-            )
-            if key is not None and key in api_secrets
+            for key in found_keys
+            if key and key in api_secrets
         )
 
     def geoip(
@@ -660,12 +724,18 @@ class BaseRequestHandler(RequestHandler):
 class HTMLRequestHandler(BaseRequestHandler):
     """A request handler that serves HTML."""
 
+    POSSIBLE_CONTENT_TYPES: tuple[str, ...] = (
+        "text/html",
+        "text/plain",
+        "text/markdown",
+        "application/json",
+    )
     used_render = False
 
     def get_form_appendix(self) -> str:
         """Get HTML to add to forms to keep important query args."""
         form_appendix = (
-            "<input name='no_3rd_party' class='hidden-input' "
+            "<input name='no_3rd_party' class='hidden' "
             f"value='{bool_to_str(self.get_no_3rd_party())}'>"
             if "no_3rd_party" in self.request.query_arguments
             and self.get_no_3rd_party() != self.get_saved_no_3rd_party()
@@ -673,12 +743,12 @@ class HTMLRequestHandler(BaseRequestHandler):
         )
         if self.get_dynload() != self.get_saved_dynload():
             form_appendix += (
-                "<input name='dynload' class='hidden-input' "
+                "<input name='dynload' class='hidden' "
                 f"value='{bool_to_str(self.get_dynload())}'>"
             )
         if (theme := self.get_theme()) != self.get_saved_theme():
             form_appendix += (
-                f"<input name='theme' class='hidden-input' value='{theme}'>"
+                f"<input name='theme' class='hidden' value='{theme}'>"
             )
         return form_appendix
 
@@ -734,15 +804,18 @@ class HTMLRequestHandler(BaseRequestHandler):
             or self.now.date() == date(self.now.year, 4, 1)
             or str_to_bool(self.get_cookie("c", "f") or "f", False),
             dynload=self.get_dynload(),
-            as_json=self.get_as_json(),
+            as_html=self.content_type == "text/html",
             now=self.now,
         )
-        namespace["🥚"] = (
-            self.settings.get("TESTING")
-            or timedelta()
-            <= self.now.date() - easter(self.now.year)
-            < timedelta(days=2)
-            or isprime(self.now.microsecond)  # type: ignore[no-untyped-call]
+        namespace.update(
+            {
+                "🥚": self.settings.get("TESTING")
+                or timedelta()
+                <= self.now.date() - easter(self.now.year)
+                < timedelta(days=2),
+                "🦘": self.settings.get("TESTING")
+                or isprime(self.now.microsecond),  # type: ignore[no-untyped-call]
+            }
         )
         return namespace
 
@@ -755,21 +828,35 @@ class HTMLRequestHandler(BaseRequestHandler):
         self, chunk: None | str | bytes | dict[Any, Any] = None
     ) -> Future[None]:
         """Finish the request."""
+        as_json: bool = self.content_type == "application/json"
+        as_plain_text: bool = self.content_type == "text/plain"
+        as_markdown: bool = self.content_type == "text/markdown"
         if (
-            isinstance(chunk, dict)
+            not isinstance(chunk, bytes | str)
             or chunk is None
             or not self.used_render
             or getattr(self, "IS_NOT_HTML", False)
-            or not self.get_as_json()
+            or not (as_json or as_plain_text or as_markdown)
         ):
             return super().finish(chunk)
+
+        chunk = chunk.decode("utf-8") if isinstance(chunk, bytes) else chunk
+
+        if as_markdown:
+            self.set_header("Content-Type", "text/markdown; charset=UTF-8")
+            return super().finish(
+                f"# {self.title}\n\n"
+                + html2text.html2text(chunk, self.request.full_url()).strip()
+            )
+
+        soup = BeautifulSoup(chunk, features="lxml")
+        if as_plain_text:
+            self.set_header("Content-Type", "text/plain; charset=UTF-8")
+            return super().finish(soup.get_text("\n", True))
+
         self.set_header("Content-Type", "application/json; charset=UTF-8")
-        soup = BeautifulSoup(
-            chunk.decode("utf-8") if isinstance(chunk, bytes) else chunk,
-            features="lxml",
-        )
         dictionary: dict[str, Any] = dict(
-            url=self.fix_url(as_json=None),  # request url without as_json param
+            url=self.fix_url(),  # request url
             title=self.title,
             short_title=self.short_title
             if self.title != self.short_title
@@ -808,13 +895,12 @@ class APIRequestHandler(BaseRequestHandler):
     It overrides the write error method to return errors as JSON.
     """
 
+    POSSIBLE_CONTENT_TYPES: tuple[str, ...] = (
+        "application/json",
+        "application/yaml",
+    )
     IS_NOT_HTML = True
 
-    def set_default_headers(self) -> None:
-        """Set important default headers for the API request handlers."""
-        super().set_default_headers()
-        self.set_header("Content-Type", "application/json; charset=UTF-8")
-
     def finish_dict(self, **kwargs: Any) -> Future[None]:
         """Finish the request with a JSON response."""
         return self.finish(kwargs)
@@ -841,6 +927,8 @@ class NotFoundHandler(HTMLRequestHandler):
         # pylint: disable=attribute-defined-outside-init
         self.now = await self.get_time()  # used by get_template_namespace
 
+        self.handle_accept_header()
+
         if self.request.method not in ("GET", "HEAD"):
             raise HTTPError(404)
         new_path = (
